#include "UpDown_objects.h"
#include "UpDown_resources.h"

//--------------------- User code ---------------------//

#define WIRE_CONNECTED 0
#define WIRE_DISCONNECTED 1

#define PURPLE_WIRE TRISB.F0
#define BLUE_WIRE TRISB.F1
#define BROWN_WIRE TRISB.F2
#define RED_WIRE TRISB.F3

#define DIAL_ROTATION_DEBOUNCE_DELAY 100
#define DIAL_ROTATION_NOTCH_DELAY_MS 10

// Menu Up / Longer Exposure
void DialUp(){
  // Step 1
  BLUE_WIRE = WIRE_CONNECTED; 
  PURPLE_WIRE = WIRE_DISCONNECTED;
  Delay_us(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 2
  BLUE_WIRE = WIRE_CONNECTED; 
  PURPLE_WIRE = WIRE_CONNECTED;
  Delay_us(DIAL_ROTATION_DEBOUNCE_DELAY);
  
  // Step 3
  BLUE_WIRE = WIRE_DISCONNECTED; 
  PURPLE_WIRE = WIRE_CONNECTED;
  Delay_us(DIAL_ROTATION_DEBOUNCE_DELAY);
  
  // Step 4
  BLUE_WIRE = WIRE_DISCONNECTED; 
  PURPLE_WIRE = WIRE_DISCONNECTED;
  Delay_us(DIAL_ROTATION_DEBOUNCE_DELAY);
  
  // Wait for a single scroll cycle pause
  Delay_ms(DIAL_ROTATION_NOTCH_DELAY_MS);
}

// Menu Down / Shorter Exposure
void DialDown(){
  // Step 1
  BLUE_WIRE = WIRE_DISCONNECTED;
  PURPLE_WIRE = WIRE_CONNECTED;
  Delay_us(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 2
  BLUE_WIRE = WIRE_CONNECTED;
  PURPLE_WIRE = WIRE_CONNECTED;
  Delay_us(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 3
  BLUE_WIRE = WIRE_CONNECTED;
  PURPLE_WIRE = WIRE_DISCONNECTED;
  Delay_us(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 4
  BLUE_WIRE = WIRE_DISCONNECTED;
  PURPLE_WIRE = WIRE_DISCONNECTED;
  Delay_us(DIAL_ROTATION_DEBOUNCE_DELAY);
  
  // Wait for a single scroll cycle pause
  Delay_ms(DIAL_ROTATION_NOTCH_DELAY_MS);
}

//----------------- End of User code ------------------//

// Event Handlers



void ShutterSlowerButtonOnPress() {
  // Menu Down / Shorter Exposure
  DialDown();
}

void ShutterFasterButtonOnPress() {
  // Menu Up / Longer Exposure
  DialUp();
}

void FullResetbuttonOnClick() {
  // Reset the exposure speed all the way back to 1/4000th sec
  int i = 0;

  for(i=0;i<=36;i++){
    DialUp();
    
    // Wait for a single scroll cycle pause
    Delay_ms(DIAL_ROTATION_NOTCH_DELAY_MS);
  }
}

void StopMinusButtonRoundOnClick() {
  // Move down one full stop
  DialDown();
  DialDown();
}

void StopPlusButtonRoundOnClick() {
  // Move up one full stop
  DialUp();
  DialUp();
}

void ShutterButtonOnClick() {
  // Update the shutter button
  ShutterButton.Gradient_Start_Color = CL_LIME;
  ShutterButton.Gradient_End_Color = CL_GREEN;
  // Refresh the shutter button
  DrawCircleButton(&ShutterButton);
  
  // Press the shutter button
  BROWN_WIRE = WIRE_CONNECTED;
  Delay_ms(100);
  
  // Release the shutter button
  BROWN_WIRE = WIRE_DISCONNECTED;
  Delay_ms(100);
  
  ShutterButton.Gradient_Start_Color = CL_RED;
  ShutterButton.Gradient_End_Color = CL_MAROON;
  // Refresh the shutter button
  DrawCircleButton(&ShutterButton);
}
