#include "hdr_clicker_objects.h"
#include "hdr_clicker_resources.h"
#include "built_in.h"


// TFT module connections
char TFT_DataPort at LATE;
sbit TFT_RST at LATC1_bit;
sbit TFT_BLED at LATA9_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATF12_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_BLED_Direction at TRISA9_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISF12_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at LATB13_bit;
sbit DriveX_Right at LATB11_bit;
sbit DriveY_Up at LATB12_bit;
sbit DriveY_Down at LATB10_bit;
sbit DriveX_Left_Direction at TRISB13_bit;
sbit DriveX_Right_Direction at TRISB11_bit;
sbit DriveY_Up_Direction at TRISB12_bit;
sbit DriveY_Down_Direction at TRISB10_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 1000;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TButton *local_button;
TButton *exec_button;
int button_order;
TButton_Round *local_round_button;
TButton_Round *exec_round_button;
int round_button_order;
TLabel *local_label;
TLabel *exec_label;
int label_order;
TBox *local_box;
TBox *exec_box;
int box_order;


void PMPWaitBusy() {
  while(PMMODEbits.BUSY);
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  PMDIN = index;
  PMPWaitBusy();
}

void Write_Command( unsigned short cmd ) {
  TFT_RS = 1;
  PMDIN = cmd;
  PMPWaitBusy();
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  PMDIN = _data;
  PMPWaitBusy();
}


void Init_ADC() {
  AD1PCFG = 0xFFFF;
  PCFG12_bit = 0;
  PCFG13_bit = 0;
  // PMP setup
  ADC1_Init();
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  
  // Flip the screen 180
  TFT_Rotate_180(1);
  TP_TFT_Rotate_180(1);
  //TP_TFT_Rotate(1);
  
  TFT_Init(320, 240);
  TP_TFT_Init(320, 240, 13, 12);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  
  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                HdrScreen;
  TButton_Round          BracketButton;
char BracketButton_Caption[16] = "9 Stops";

  TButton_Round          HelpScreenButton;
char HelpScreenButton_Caption[5] = "Help";

  TButton_Round          ShootButton;
char ShootButton_Caption[11] = "Shoot  HDR";

  TButton_Round          * const code Screen1_Buttons_Round[3]=
         {
         &BracketButton,       
         &HelpScreenButton,    
         &ShootButton          
         };

  TScreen                HelpScreen;
  TLabel                 Label1;
char Label1_Caption[19] = "Setup Instructions";

  TLabel                 Label2;
char Label2_Caption[26] = "Turn on MCU before camera";

  TLabel                 Label3;
char Label3_Caption[18] = "Drive: Continuous";

  TLabel                 Label5;
char Label5_Caption[16] = "Camera Settings";

  TLabel                 Label6;
char Label6_Caption[19] = "White Balance: Sun";

  TLabel                 Label7;
char Label7_Caption[20] = "Quality: JPEG Large";

  TLabel                 Label8;
char Label8_Caption[18] = "RB0: Blue Dial Up";

  TLabel                 Label4;
char Label4_Caption[11] = "GND: Green";

  TLabel                 Label9;
char Label9_Caption[9] = "ISO: 100";

  TLabel                 Label10;
char Label10_Caption[12] = "Review: Off";

  TLabel                 Label11;
char Label11_Caption[21] = "Erase Camera CF Card";

  TLabel                 Label12;
char Label12_Caption[16] = "Focus: Infinity";

  TLabel                 Label13;
char Label13_Caption[22] = "Aperture: f/16 Locked";

  TButton               BackButton;
char BackButton_Caption[5] = "Back";

  TLabel                 Label14;
char Label14_Caption[22] = "RB1: Purple Dial Down";

  TLabel                 Label15;
char Label15_Caption[19] = "RB2: Brown Shutter";

  TLabel                 Label16;
char Label16_Caption[15] = "RB3: Red Focus";

  TBox                   BackButtonBox;
  TLabel                 Label17;
char Label17_Caption[14] = "AEB: One Stop";

  TButton                * const code Screen2_Buttons[1]=
         {
         &BackButton           
         };
  TLabel                 * const code Screen2_Labels[17]=
         {
         &Label1,              
         &Label2,              
         &Label3,              
         &Label5,              
         &Label6,              
         &Label7,              
         &Label8,              
         &Label4,              
         &Label9,              
         &Label10,             
         &Label11,             
         &Label12,             
         &Label13,             
         &Label14,             
         &Label15,             
         &Label16,             
         &Label17              
         };
  TBox                   * const code Screen2_Boxes[1]=
         {
         &BackButtonBox        
         };



static void InitializeObjects() {
  HdrScreen.Color                     = 0x2965;
  HdrScreen.Width                     = 320;
  HdrScreen.Height                    = 240;
  HdrScreen.ButtonsCount              = 0;
  HdrScreen.Buttons_RoundCount        = 3;
  HdrScreen.Buttons_Round             = Screen1_Buttons_Round;
  HdrScreen.LabelsCount               = 0;
  HdrScreen.BoxesCount                = 0;
  HdrScreen.ObjectsCount              = 3;

  HelpScreen.Color                     = 0x4228;
  HelpScreen.Width                     = 320;
  HelpScreen.Height                    = 240;
  HelpScreen.ButtonsCount              = 1;
  HelpScreen.Buttons                   = Screen2_Buttons;
  HelpScreen.Buttons_RoundCount        = 0;
  HelpScreen.LabelsCount               = 17;
  HelpScreen.Labels                    = Screen2_Labels;
  HelpScreen.BoxesCount                = 1;
  HelpScreen.Boxes                     = Screen2_Boxes;
  HelpScreen.ObjectsCount              = 19;


  BracketButton.OwnerScreen     = &HdrScreen;
  BracketButton.Order           = 0;
  BracketButton.Left            = 8;
  BracketButton.Top             = 8;
  BracketButton.Width           = 64;
  BracketButton.Height          = 28;
  BracketButton.Pen_Width       = 1;
  BracketButton.Pen_Color       = 0x0000;
  BracketButton.Visible         = 1;
  BracketButton.Active          = 1;
  BracketButton.Transparent     = 1;
  BracketButton.Caption         = BracketButton_Caption;
  BracketButton.TextAlign       = _taCenter;
  BracketButton.FontName        = Tahoma15x18_Regular;
  BracketButton.PressColEnabled = 1;
  BracketButton.Font_Color      = 0x0000;
  BracketButton.Gradient        = 1;
  BracketButton.Gradient_Orientation = 0;
  BracketButton.Gradient_Start_Color = 0xFFFF;
  BracketButton.Gradient_End_Color = 0xC618;
  BracketButton.Color           = 0xC618;
  BracketButton.Press_Color     = 0xE71C;
  BracketButton.Corner_Radius   = 3;
  BracketButton.OnUpPtr         = 0;
  BracketButton.OnDownPtr       = 0;
  BracketButton.OnClickPtr      = BracketButtonOnClick;
  BracketButton.OnPressPtr      = 0;

  HelpScreenButton.OwnerScreen     = &HdrScreen;
  HelpScreenButton.Order           = 1;
  HelpScreenButton.Left            = 248;
  HelpScreenButton.Top             = 8;
  HelpScreenButton.Width           = 64;
  HelpScreenButton.Height          = 28;
  HelpScreenButton.Pen_Width       = 1;
  HelpScreenButton.Pen_Color       = 0x0000;
  HelpScreenButton.Visible         = 1;
  HelpScreenButton.Active          = 1;
  HelpScreenButton.Transparent     = 1;
  HelpScreenButton.Caption         = HelpScreenButton_Caption;
  HelpScreenButton.TextAlign       = _taCenter;
  HelpScreenButton.FontName        = Tahoma15x18_Regular;
  HelpScreenButton.PressColEnabled = 1;
  HelpScreenButton.Font_Color      = 0x0000;
  HelpScreenButton.Gradient        = 1;
  HelpScreenButton.Gradient_Orientation = 0;
  HelpScreenButton.Gradient_Start_Color = 0xFFFF;
  HelpScreenButton.Gradient_End_Color = 0xC618;
  HelpScreenButton.Color           = 0xC618;
  HelpScreenButton.Press_Color     = 0xE71C;
  HelpScreenButton.Corner_Radius   = 3;
  HelpScreenButton.OnUpPtr         = 0;
  HelpScreenButton.OnDownPtr       = 0;
  HelpScreenButton.OnClickPtr      = HelpScreenButtonOnClick;
  HelpScreenButton.OnPressPtr      = 0;

  ShootButton.OwnerScreen     = &HdrScreen;
  ShootButton.Order           = 2;
  ShootButton.Left            = 15;
  ShootButton.Top             = 74;
  ShootButton.Width           = 294;
  ShootButton.Height          = 152;
  ShootButton.Pen_Width       = 5;
  ShootButton.Pen_Color       = 0x0000;
  ShootButton.Visible         = 1;
  ShootButton.Active          = 1;
  ShootButton.Transparent     = 1;
  ShootButton.Caption         = ShootButton_Caption;
  ShootButton.TextAlign       = _taCenter;
  ShootButton.FontName        = Tahoma56x57_Bold;
  ShootButton.PressColEnabled = 1;
  ShootButton.Font_Color      = 0xFFFF;
  ShootButton.Gradient        = 1;
  ShootButton.Gradient_Orientation = 0;
  ShootButton.Gradient_Start_Color = 0xF800;
  ShootButton.Gradient_End_Color = 0x8000;
  ShootButton.Color           = 0x8000;
  ShootButton.Press_Color     = 0xE71C;
  ShootButton.Corner_Radius   = 3;
  ShootButton.OnUpPtr         = 0;
  ShootButton.OnDownPtr       = 0;
  ShootButton.OnClickPtr      = ShootButtonOnClick;
  ShootButton.OnPressPtr      = 0;

  Label1.OwnerScreen     = &HelpScreen;
  Label1.Order           = 0;
  Label1.Left            = 73;
  Label1.Top             = 4;
  Label1.Width           = 166;
  Label1.Height          = 27;
  Label1.Visible         = 1;
  Label1.Active          = 1;
  Label1.Caption         = Label1_Caption;
  Label1.FontName        = Tahoma20x24_Regular;
  Label1.Font_Color      = 0xFFFF;
  Label1.OnUpPtr         = 0;
  Label1.OnDownPtr       = 0;
  Label1.OnClickPtr      = 0;
  Label1.OnPressPtr      = 0;

  Label2.OwnerScreen     = &HelpScreen;
  Label2.Order           = 1;
  Label2.Left            = 144;
  Label2.Top             = 30;
  Label2.Width           = 167;
  Label2.Height          = 17;
  Label2.Visible         = 1;
  Label2.Active          = 1;
  Label2.Caption         = Label2_Caption;
  Label2.FontName        = Tahoma12x16_Regular;
  Label2.Font_Color      = 0xFFFF;
  Label2.OnUpPtr         = 0;
  Label2.OnDownPtr       = 0;
  Label2.OnClickPtr      = 0;
  Label2.OnPressPtr      = 0;

  Label3.OwnerScreen     = &HelpScreen;
  Label3.Order           = 2;
  Label3.Left            = 10;
  Label3.Top             = 146;
  Label3.Width           = 106;
  Label3.Height          = 17;
  Label3.Visible         = 1;
  Label3.Active          = 1;
  Label3.Caption         = Label3_Caption;
  Label3.FontName        = Tahoma12x16_Regular;
  Label3.Font_Color      = 0xFFFF;
  Label3.OnUpPtr         = 0;
  Label3.OnDownPtr       = 0;
  Label3.OnClickPtr      = 0;
  Label3.OnPressPtr      = 0;

  Label5.OwnerScreen     = &HelpScreen;
  Label5.Order           = 3;
  Label5.Left            = 72;
  Label5.Top             = 119;
  Label5.Width           = 149;
  Label5.Height          = 27;
  Label5.Visible         = 1;
  Label5.Active          = 1;
  Label5.Caption         = Label5_Caption;
  Label5.FontName        = Tahoma20x24_Regular;
  Label5.Font_Color      = 0xFFFF;
  Label5.OnUpPtr         = 0;
  Label5.OnDownPtr       = 0;
  Label5.OnClickPtr      = 0;
  Label5.OnPressPtr      = 0;

  Label6.OwnerScreen     = &HelpScreen;
  Label6.Order           = 4;
  Label6.Left            = 10;
  Label6.Top             = 163;
  Label6.Width           = 115;
  Label6.Height          = 17;
  Label6.Visible         = 1;
  Label6.Active          = 1;
  Label6.Caption         = Label6_Caption;
  Label6.FontName        = Tahoma12x16_Regular;
  Label6.Font_Color      = 0xFFFF;
  Label6.OnUpPtr         = 0;
  Label6.OnDownPtr       = 0;
  Label6.OnClickPtr      = 0;
  Label6.OnPressPtr      = 0;

  Label7.OwnerScreen     = &HelpScreen;
  Label7.Order           = 5;
  Label7.Left            = 10;
  Label7.Top             = 180;
  Label7.Width           = 117;
  Label7.Height          = 17;
  Label7.Visible         = 1;
  Label7.Active          = 1;
  Label7.Caption         = Label7_Caption;
  Label7.FontName        = Tahoma12x16_Regular;
  Label7.Font_Color      = 0xFFFF;
  Label7.OnUpPtr         = 0;
  Label7.OnDownPtr       = 0;
  Label7.OnClickPtr      = 0;
  Label7.OnPressPtr      = 0;

  Label8.OwnerScreen     = &HelpScreen;
  Label8.Order           = 6;
  Label8.Left            = 11;
  Label8.Top             = 48;
  Label8.Width           = 105;
  Label8.Height          = 17;
  Label8.Visible         = 1;
  Label8.Active          = 1;
  Label8.Caption         = Label8_Caption;
  Label8.FontName        = Tahoma12x16_Regular;
  Label8.Font_Color      = 0x041F;
  Label8.OnUpPtr         = 0;
  Label8.OnDownPtr       = 0;
  Label8.OnClickPtr      = 0;
  Label8.OnPressPtr      = 0;

  Label4.OwnerScreen     = &HelpScreen;
  Label4.Order           = 7;
  Label4.Left            = 11;
  Label4.Top             = 30;
  Label4.Width           = 71;
  Label4.Height          = 17;
  Label4.Visible         = 1;
  Label4.Active          = 1;
  Label4.Caption         = Label4_Caption;
  Label4.FontName        = Tahoma12x16_Regular;
  Label4.Font_Color      = 0x07E0;
  Label4.OnUpPtr         = 0;
  Label4.OnDownPtr       = 0;
  Label4.OnClickPtr      = 0;
  Label4.OnPressPtr      = 0;

  Label9.OwnerScreen     = &HelpScreen;
  Label9.Order           = 8;
  Label9.Left            = 10;
  Label9.Top             = 198;
  Label9.Width           = 53;
  Label9.Height          = 17;
  Label9.Visible         = 1;
  Label9.Active          = 1;
  Label9.Caption         = Label9_Caption;
  Label9.FontName        = Tahoma12x16_Regular;
  Label9.Font_Color      = 0xFFFF;
  Label9.OnUpPtr         = 0;
  Label9.OnDownPtr       = 0;
  Label9.OnClickPtr      = 0;
  Label9.OnPressPtr      = 0;

  Label10.OwnerScreen     = &HelpScreen;
  Label10.Order           = 9;
  Label10.Left            = 10;
  Label10.Top             = 215;
  Label10.Width           = 69;
  Label10.Height          = 17;
  Label10.Visible         = 1;
  Label10.Active          = 1;
  Label10.Caption         = Label10_Caption;
  Label10.FontName        = Tahoma12x16_Regular;
  Label10.Font_Color      = 0xFFFF;
  Label10.OnUpPtr         = 0;
  Label10.OnDownPtr       = 0;
  Label10.OnClickPtr      = 0;
  Label10.OnPressPtr      = 0;

  Label11.OwnerScreen     = &HelpScreen;
  Label11.Order           = 10;
  Label11.Left            = 144;
  Label11.Top             = 48;
  Label11.Width           = 132;
  Label11.Height          = 17;
  Label11.Visible         = 1;
  Label11.Active          = 1;
  Label11.Caption         = Label11_Caption;
  Label11.FontName        = Tahoma12x16_Regular;
  Label11.Font_Color      = 0xFFFF;
  Label11.OnUpPtr         = 0;
  Label11.OnDownPtr       = 0;
  Label11.OnClickPtr      = 0;
  Label11.OnPressPtr      = 0;

  Label12.OwnerScreen     = &HelpScreen;
  Label12.Order           = 11;
  Label12.Left            = 144;
  Label12.Top             = 147;
  Label12.Width           = 84;
  Label12.Height          = 17;
  Label12.Visible         = 1;
  Label12.Active          = 1;
  Label12.Caption         = Label12_Caption;
  Label12.FontName        = Tahoma12x16_Regular;
  Label12.Font_Color      = 0xFFFF;
  Label12.OnUpPtr         = 0;
  Label12.OnDownPtr       = 0;
  Label12.OnClickPtr      = 0;
  Label12.OnPressPtr      = 0;

  Label13.OwnerScreen     = &HelpScreen;
  Label13.Order           = 12;
  Label13.Left            = 144;
  Label13.Top             = 164;
  Label13.Width           = 129;
  Label13.Height          = 17;
  Label13.Visible         = 1;
  Label13.Active          = 1;
  Label13.Caption         = Label13_Caption;
  Label13.FontName        = Tahoma12x16_Regular;
  Label13.Font_Color      = 0xFFFF;
  Label13.OnUpPtr         = 0;
  Label13.OnDownPtr       = 0;
  Label13.OnClickPtr      = 0;
  Label13.OnPressPtr      = 0;

  BackButton.OwnerScreen     = &HelpScreen;
  BackButton.Order           = 13;
  BackButton.Left            = 264;
  BackButton.Top             = 206;
  BackButton.Width           = 48;
  BackButton.Height          = 23;
  BackButton.Pen_Width       = 1;
  BackButton.Pen_Color       = 0x0000;
  BackButton.Visible         = 1;
  BackButton.Active          = 1;
  BackButton.Transparent     = 1;
  BackButton.Caption         = BackButton_Caption;
  BackButton.TextAlign       = _taCenter;
  BackButton.FontName        = Tahoma12x14_Regular;
  BackButton.PressColEnabled = 1;
  BackButton.Font_Color      = 0x0000;
  BackButton.Gradient        = 1;
  BackButton.Gradient_Orientation = 0;
  BackButton.Gradient_Start_Color = 0xFFFF;
  BackButton.Gradient_End_Color = 0xC618;
  BackButton.Color           = 0xC618;
  BackButton.Press_Color     = 0xE71C;
  BackButton.OnUpPtr         = 0;
  BackButton.OnDownPtr       = 0;
  BackButton.OnClickPtr      = BackButtonOnClick;
  BackButton.OnPressPtr      = 0;

  Label14.OwnerScreen     = &HelpScreen;
  Label14.Order           = 14;
  Label14.Left            = 11;
  Label14.Top             = 65;
  Label14.Width           = 134;
  Label14.Height          = 17;
  Label14.Visible         = 1;
  Label14.Active          = 1;
  Label14.Caption         = Label14_Caption;
  Label14.FontName        = Tahoma12x16_Regular;
  Label14.Font_Color      = 0xB31F;
  Label14.OnUpPtr         = 0;
  Label14.OnDownPtr       = 0;
  Label14.OnClickPtr      = 0;
  Label14.OnPressPtr      = 0;

  Label15.OwnerScreen     = &HelpScreen;
  Label15.Order           = 15;
  Label15.Left            = 11;
  Label15.Top             = 82;
  Label15.Width           = 117;
  Label15.Height          = 17;
  Label15.Visible         = 1;
  Label15.Active          = 1;
  Label15.Caption         = Label15_Caption;
  Label15.FontName        = Tahoma12x16_Regular;
  Label15.Font_Color      = 0xD280;
  Label15.OnUpPtr         = 0;
  Label15.OnDownPtr       = 0;
  Label15.OnClickPtr      = 0;
  Label15.OnPressPtr      = 0;

  Label16.OwnerScreen     = &HelpScreen;
  Label16.Order           = 16;
  Label16.Left            = 11;
  Label16.Top             = 100;
  Label16.Width           = 93;
  Label16.Height          = 17;
  Label16.Visible         = 1;
  Label16.Active          = 1;
  Label16.Caption         = Label16_Caption;
  Label16.FontName        = Tahoma12x16_Regular;
  Label16.Font_Color      = 0xF800;
  Label16.OnUpPtr         = 0;
  Label16.OnDownPtr       = 0;
  Label16.OnClickPtr      = 0;
  Label16.OnPressPtr      = 0;

  BackButtonBox.OwnerScreen     = &HelpScreen;
  BackButtonBox.Order           = 17;
  BackButtonBox.Left            = 0;
  BackButtonBox.Top             = 0;
  BackButtonBox.Width           = 320;
  BackButtonBox.Height          = 240;
  BackButtonBox.Pen_Width       = 0;
  BackButtonBox.Pen_Color       = 0x0000;
  BackButtonBox.Visible         = 1;
  BackButtonBox.Active          = 1;
  BackButtonBox.Transparent     = 0;
  BackButtonBox.Gradient        = 0;
  BackButtonBox.Gradient_Orientation = 0;
  BackButtonBox.Gradient_Start_Color = 0xFFFF;
  BackButtonBox.Gradient_End_Color = 0xC618;
  BackButtonBox.Color           = 0xC618;
  BackButtonBox.PressColEnabled = 1;
  BackButtonBox.Press_Color     = 0xE71C;
  BackButtonBox.OnUpPtr         = 0;
  BackButtonBox.OnDownPtr       = 0;
  BackButtonBox.OnClickPtr      = BackButtonBoxOnClick;
  BackButtonBox.OnPressPtr      = 0;

  Label17.OwnerScreen     = &HelpScreen;
  Label17.Order           = 18;
  Label17.Left            = 143;
  Label17.Top             = 182;
  Label17.Width           = 87;
  Label17.Height          = 17;
  Label17.Visible         = 1;
  Label17.Active          = 1;
  Label17.Caption         = Label17_Caption;
  Label17.FontName        = Tahoma12x16_Regular;
  Label17.Font_Color      = 0xFFFF;
  Label17.OnUpPtr         = 0;
  Label17.OnDownPtr       = 0;
  Label17.OnClickPtr      = 0;
  Label17.OnPressPtr      = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetButton(index)              CurrentScreen->Buttons[index]
#define GetRoundButton(index)         CurrentScreen->Buttons_Round[index]
#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]


void DrawButton(TButton *Abutton) {
  if (Abutton->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(Abutton->Transparent, Abutton->Press_Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_End_Color, Abutton->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(Abutton->Transparent, Abutton->Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_Start_Color, Abutton->Gradient_End_Color);
    }
    TFT_Set_Pen(Abutton->Pen_Color, Abutton->Pen_Width);
    TFT_Rectangle(Abutton->Left, Abutton->Top, Abutton->Left + Abutton->Width - 1, Abutton->Top + Abutton->Height - 1);
    TFT_Set_Font(Abutton->FontName, Abutton->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    if (Abutton->TextAlign == _taLeft)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + 4, (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taCenter)
      TFT_Write_Text(Abutton->Caption, (Abutton->Left + (Abutton->Width - caption_length) / 2), (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taRight)
      TFT_Write_Text(Abutton->Caption, Abutton->Left + (Abutton->Width - caption_length - 4), (Abutton->Top + (Abutton->Height - caption_height) / 2));
  }
}

void DrawRoundButton(TButton_Round *Around_button) {
    if (Around_button->Visible == 1) {
      if (object_pressed == 1) {
        object_pressed = 0;
        TFT_Set_Brush(Around_button->Transparent, Around_button->Press_Color, Around_button->Gradient, Around_button->Gradient_Orientation,
                      Around_button->Gradient_End_Color, Around_button->Gradient_Start_Color);
      }
      else {
        TFT_Set_Brush(Around_button->Transparent, Around_button->Color, Around_button->Gradient, Around_button->Gradient_Orientation,
                      Around_button->Gradient_Start_Color, Around_button->Gradient_End_Color);
      }
      TFT_Set_Pen(Around_button->Pen_Color, Around_button->Pen_Width);
      TFT_Rectangle_Round_Edges(Around_button->Left + 1, Around_button->Top + 1,
        Around_button->Left + Around_button->Width - 2,
        Around_button->Top + Around_button->Height - 2, Around_button->Corner_Radius);
      TFT_Set_Font(Around_button->FontName, Around_button->Font_Color, FO_HORIZONTAL);
      TFT_Write_Text_Return_Pos(Around_button->Caption, Around_button->Left, Around_button->Top);
      if (Around_button->TextAlign == _taLeft)
        TFT_Write_Text(Around_button->Caption, Around_button->Left + 4, (Around_button->Top + ((Around_button->Height - caption_height) / 2)));
      else if (Around_button->TextAlign == _taCenter)
        TFT_Write_Text(Around_button->Caption, (Around_button->Left + (Around_button->Width - caption_length) / 2), (Around_button->Top + ((Around_button->Height - caption_height) / 2)));
      else if (Around_button->TextAlign == _taRight)
        TFT_Write_Text(Around_button->Caption, Around_button->Left + (Around_button->Width - caption_length - 4), (Around_button->Top + (Around_button->Height - caption_height) / 2));
    }
}

void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(ABox->Transparent, ABox->Press_Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_End_Color, ABox->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    }
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short button_idx;
  TButton *local_button;
  unsigned short round_button_idx;
  TButton_Round *local_round_button;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short box_idx;
  TBox *local_box;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  button_idx = 0;
  round_button_idx = 0;
  label_idx = 0;
  box_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    save_bled_direction = TFT_BLED_Direction;
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Set_Active(Set_Index, Write_Command, Write_Data);
    TFT_Init(CurrentScreen->Width, CurrentScreen->Height);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 13, 12);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
    TFT_BLED_Direction = save_bled_direction;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (button_idx < CurrentScreen->ButtonsCount) {
      local_button = GetButton(button_idx);
      if (order == local_button->Order) {
        button_idx++;
        order++;
        DrawButton(local_button);
      }
    }

    if (round_button_idx < CurrentScreen->Buttons_RoundCount) {
      local_round_button = GetRoundButton(round_button_idx);
      if (order == local_round_button->Order) {
        round_button_idx++;
        order++;
        DrawRoundButton(local_round_button);
      }
    }

    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  button_order        = -1;
  round_button_order  = -1;
  label_order         = -1;
  box_order           = -1;
  //  Buttons
  for ( _object_count = 0 ; _object_count < CurrentScreen->ButtonsCount ; _object_count++ ) {
    local_button = GetButton(_object_count);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  //  Buttons with Round Edges
  for ( _object_count = 0 ; _object_count < CurrentScreen->Buttons_RoundCount ; _object_count++ ) {
    local_round_button = GetRoundButton(_object_count);
    if (local_round_button->Active == 1) {
      if (IsInsideObject(X, Y, local_round_button->Left, local_round_button->Top,
                         local_round_button->Width, local_round_button->Height) == 1) {
        round_button_order = local_round_button->Order;
        exec_round_button = local_round_button;
      }
    }
  }

  //  Labels
  for ( _object_count = 0 ; _object_count < CurrentScreen->LabelsCount ; _object_count++ ) {
    local_label = GetLabel(_object_count);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  //  Boxes
  for ( _object_count = 0 ; _object_count < CurrentScreen->BoxesCount ; _object_count++ ) {
    local_box = GetBox(_object_count);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  _object_count = -1;
  if (button_order >  _object_count )
    _object_count = button_order;
  if (round_button_order >  _object_count )
    _object_count = round_button_order;
  if (label_order >  _object_count )
    _object_count = label_order;
  if (box_order >  _object_count )
    _object_count = box_order;
}


void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_button         = 0;
  exec_round_button   = 0;
  exec_label          = 0;
  exec_box            = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnPressPtr != 0) {
          exec_button->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == round_button_order) {
      if (exec_round_button->Active == 1) {
        if (exec_round_button->OnPressPtr != 0) {
          exec_round_button->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Button
    case 0: {
      if (PressedObject != 0) {
        exec_button = (TButton*)PressedObject;
        if ((exec_button->PressColEnabled == 1) && (exec_button->OwnerScreen == CurrentScreen)) {
          DrawButton(exec_button);
        }
        break;
      }
      break;
    }
    // Round Button
    case 1: {
      if (PressedObject != 0) {
        exec_round_button = (TButton_Round*)PressedObject;
        if ((exec_round_button->PressColEnabled == 1) && (exec_round_button->OwnerScreen == CurrentScreen)) {
          DrawRoundButton(exec_round_button);
        }
        break;
      }
      break;
    }
    // Box
    case 6: {
      if (PressedObject != 0) {
        exec_box = (TBox*)PressedObject;
        if ((exec_box->PressColEnabled == 1) && (exec_box->OwnerScreen == CurrentScreen)) {
          DrawBox(exec_box);
        }
        break;
      }
      break;
    }
  }

  exec_label          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Buttons
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnUpPtr != 0)
          exec_button->OnUpPtr();
        if (PressedObject == (void *)exec_button)
          if (exec_button->OnClickPtr != 0)
            exec_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Buttons with Round Edges
    if (_object_count == round_button_order) {
      if (exec_round_button->Active == 1) {
        if (exec_round_button->OnUpPtr != 0)
          exec_round_button->OnUpPtr();
        if (PressedObject == (void *)exec_round_button)
          if (exec_round_button->OnClickPtr != 0)
            exec_round_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Labels
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes
    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (void *)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_button         = 0;
  exec_round_button   = 0;
  exec_label          = 0;
  exec_box            = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->PressColEnabled == 1) {
          object_pressed = 1;
          DrawButton(exec_button);
        }
        PressedObject = (void *)exec_button;
        PressedObjectType = 0;
        if (exec_button->OnDownPtr != 0) {
          exec_button->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == round_button_order) {
      if (exec_round_button->Active == 1) {
        if (exec_round_button->PressColEnabled == 1) {
          object_pressed = 1;
          DrawRoundButton(exec_round_button);
        }
        PressedObject = (void *)exec_round_button;
        PressedObjectType = 1;
        if (exec_round_button->OnDownPtr != 0) {
          exec_round_button->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (void *)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->PressColEnabled == 1) {
          object_pressed = 1;
          DrawBox(exec_box);
        }
        PressedObject = (void *)exec_box;
        PressedObjectType = 6;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  PMMODE = 0;
  PMAEN  = 0;
  PMCON  = 0;  // WRSP: Write Strobe Polarity bit
  PMMODEbits.MODE = 2;     // Master 2
  PMMODEbits.WAITB = 0;
  PMMODEbits.WAITM = 1;
  PMMODEbits.WAITE = 0;
  PMMODEbits.MODE16 = 1;   // 16 bit mode
  PMCONbits.CSF = 0;
  PMCONbits.PTRDEN = 1;
  PMCONbits.PTWREN = 1;
  PMCONbits.PMPEN = 1;
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(76, 907, 77, 915);    // Set calibration constants

  InitializeObjects();
  display_width = HdrScreen.Width;
  display_height = HdrScreen.Height;
  DrawScreen(&HdrScreen);
}