#include "hdr_clicker_objects.h"
#include "hdr_clicker_resources.h"

//--------------------- User code ---------------------//

void DialUp();
void DialDown();
void AEBDialDown();
void DialFullReset();
void DialShortReset();
void ResetGuiElements();
void TakePhoto( char * caption_text);
void Check_Remote();

int bracket_steps = 9;

#define WIRE_CONNECTED 0
#define WIRE_DISCONNECTED 1

#define PURPLE_WIRE TRISB.F0
#define BLUE_WIRE TRISB.F1
#define BROWN_WIRE TRISB.F2
#define RED_WIRE TRISB.F3

// This delay MUST be at least 10 MS or the dial changes will slip
#define DIAL_ROTATION_DEBOUNCE_DELAY 12

// TC80N3 Remote input RD0
// void Check_Remote(){
  // if (Button(&PORTD, 0, 1, 0)){
    // ShootButtonOnClick();
  // }
// }

// Menu Up / Longer Exposure
void DialUp(){
  // Step 1
  BLUE_WIRE = WIRE_CONNECTED;
  PURPLE_WIRE = WIRE_DISCONNECTED;
  Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 2
  BLUE_WIRE = WIRE_CONNECTED;
  PURPLE_WIRE = WIRE_CONNECTED;
  Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 3
  BLUE_WIRE = WIRE_DISCONNECTED;
  PURPLE_WIRE = WIRE_CONNECTED;
  Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 4
  BLUE_WIRE = WIRE_DISCONNECTED;
  PURPLE_WIRE = WIRE_DISCONNECTED;
  Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);
}

// Menu Down / Shorter Exposure
void DialDown(){
  // Step 1
  BLUE_WIRE = WIRE_DISCONNECTED;
  PURPLE_WIRE = WIRE_CONNECTED;
  Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 2
  BLUE_WIRE = WIRE_CONNECTED;
  PURPLE_WIRE = WIRE_CONNECTED;
  Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 3
  BLUE_WIRE = WIRE_CONNECTED;
  PURPLE_WIRE = WIRE_DISCONNECTED;
  Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);

  // Step 4
  BLUE_WIRE = WIRE_DISCONNECTED;
  PURPLE_WIRE = WIRE_DISCONNECTED;
  Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);
}

// Move it down by 3 full stops for the next AEB bracket position
void AEBDialDown(){
  DialDown();
  DialDown();

  DialDown();
  DialDown();

  DialDown();
  DialDown();
}

void DialFullReset(){
   // Reset the exposure speed all the way back to 1/4000th sec from Bulb
  int i = 0;

  for(i=0;i<=36;i++){
    DialUp();

    // Wait for a single scroll cycle pause
    Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);
  }
  
  // Move from 1/4000th down to 1/2000th for first AEB position
  DialDown();
  DialDown();
}


void DialShortReset(){
   // Reset the exposure speed all the way back to 1/4000th sec from 1/4
  int i = 0;

  for(i=0;i<=21;i++){
    DialUp();

    // Wait for a single scroll cycle pause
    Delay_ms(DIAL_ROTATION_DEBOUNCE_DELAY);
  }
  
  // Move from 1/4000th down to 1/2000th for first AEB position
  DialDown();
  DialDown();
}


void ResetGuiElements(){
  // Update the shutter button color
  ShootButton.Gradient_Start_Color = CL_RED;
  ShootButton.Gradient_End_Color = CL_MAROON;
  strcpy(ShootButton.Caption, "Shoot  HDR");

  // Refresh the display
  DrawScreen(&HdrScreen);
}

void TakePhoto( char * caption_text){
  // Update the shutter button color
  ShootButton.Gradient_Start_Color = CL_LIME;
  ShootButton.Gradient_End_Color = CL_GREEN;
  
  // Write the shutter speed onscreen
  strcpy(ShootButton.Caption, caption_text);

  // Refresh the display
  //DrawScreen(&HdrScreen);
  DrawRoundButton(&ShootButton);
  
  // pause before the next trigger
  Delay_ms(250);
  
  // Press the shutter button
  BROWN_WIRE = WIRE_CONNECTED;
  
  // Pause while the AEB exposures are taken
  Delay_ms(1000);

  // Release the shutter button
  BROWN_WIRE = WIRE_DISCONNECTED;
  
  // 100 ms minimum delay required for logic to work in camera
  // 2000 ms is closeer to a human interaction
  Delay_ms(2000);
}


//----------------- End of User code ------------------//

// Event Handlers

void ShootButtonOnClick() {
  // Trigger 9 stops
  TakePhoto("1/2000");
  
  // Move it down by 3 full stops for the next AEB bracket position
  AEBDialDown();
  TakePhoto("1/250");
  
  // Move it down by 3 full stops for the next AEB bracket position
  AEBDialDown();
  TakePhoto("1/30");

  // Update the shutter button color
  ShootButton.Gradient_Start_Color = CL_BLUE;
  ShootButton.Gradient_End_Color = CL_NAVY;
  strcpy(ShootButton.Caption, "Disk  I/O");
  DrawRoundButton(&ShootButton);
  //DrawScreen(&HdrScreen);
  
  // flush the memory buffer to disk
  // Default 3200
  // 28 sec is full buffer empty
  Delay_ms(4000);
  
  // Reset the exposure speed all the way back to 1/4000th sec from 1/4
  DialShortReset();
  
  // update the UI back to the default shoot button
  ResetGuiElements();

}

void BracketButtonOnClick() {

  // Later we can enable one of these to offer 15 stops
  // Toggle the number of HDR stops value
  if(bracket_steps == 9){
    // 9 JPEG photos per view position with a +1/-1 EV step value
    bracket_steps = 9;
    strcpy(BracketButton.Caption, "9 Stops");
  } else {
    // 9 JPEG photos per view position with a +1/-1 EV step value
    bracket_steps = 9;
    strcpy(BracketButton.Caption, "9 Stops");
  }

  //  Refresh the bracket level button
  //DrawButton(&BracketButton);
  DrawScreen(&HdrScreen);

  // Reset the gui
  ResetGuiElements();
}

void BackButtonOnClick() {
  // Switch to the main screen
  DrawScreen(&HdrScreen);
}

void HelpScreenButtonOnClick() {
  // Switch to the help screen
  DrawScreen(&HelpScreen);
}

void BackButtonBoxOnClick() {
  // Switch to the main screen
  DrawScreen(&HdrScreen);
}